package com.ninegag.android.library.upload.media.processor

import android.content.Context
import android.net.Uri
import android.os.ParcelFileDescriptor
import com.ninegag.android.library.upload.media.validator.BaseMediaValidator
import com.ninegag.android.library.upload.model.MediaMeta
import com.under9.android.lib.util.file.SourceFileController
import jp.tomorrowkey.android.gifplayer.GifDecoder
import timber.log.Timber
import java.io.*

/**
 * Created by raymond on 9/5/2017.
 */
class GifProcessor(context: Context?,
                   sourceFileController: SourceFileController?,
                   saveMediaCallback: SaveMediaCallback?, mediaValidator: BaseMediaValidator?) : BaseProcessor(context!!, sourceFileController!!, saveMediaCallback!!, mediaValidator!!) {
    override fun getMediaMeta(f: File): MediaMeta {
        return try {
            val fis = FileInputStream(f)
            val decoder = GifDecoder()
            decoder.read(fis)
            MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_GIF)
                    .filePath(f.absolutePath)
                    .fileSize(f.length())
                    .widthAndHeight(decoder.width, decoder.height)
                    .build()
        } catch (e: Exception) {
            MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_GIF)
                    .filePath(f.absolutePath)
                    .fileSize(f.length())
                    .widthAndHeight(0, 0)
                    .build()
        }
    }

    @Throws(Exception::class)
    public override fun saveMedia(parcelFileDescriptor: ParcelFileDescriptor, contentUri: Uri, tmpFileLocation: String): MediaMeta {
        val fos = FileOutputStream(tmpFileLocation)
        val fis = FileInputStream(parcelFileDescriptor.fileDescriptor)
        val decoder = GifDecoder()
        val buf = ByteArray(1024)

        var len: Int
        while (fis.read(buf).also { len = it } > 0) {
            fos.write(buf, 0, len)
        }

        val decodeStream: InputStream = FileInputStream(tmpFileLocation)
        decoder.read(decodeStream)

        fos.close()
        return MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_GIF)
                .filePath(tmpFileLocation)
                .fileSize(fis.channel.size())
                .widthAndHeight(decoder.width, decoder.height)
                .build()
    }

    @Throws(IOException::class, NullPointerException::class)
    override fun saveMedia(mediaMeta: MediaMeta, tmpFileLocation: String) {
        val fos = FileOutputStream(tmpFileLocation)
        val fis = FileInputStream(mediaMeta.filePath)
        Timber.d("saveMedia: " + mediaMeta.filePath)
        val buf = ByteArray(1024)
        var len: Int
        while (fis.read(buf).also { len = it } > 0) {
            fos.write(buf, 0, len)
        }
        fis.close()
        fos.close()
    }
}