package com.ninegag.android.library.upload.media.processor

import android.content.Context
import android.graphics.Bitmap
import android.net.Uri
import android.os.ParcelFileDescriptor

import com.ninegag.android.library.upload.Constant
import com.ninegag.android.library.upload.media.validator.BaseMediaValidator
import com.ninegag.android.library.upload.model.MediaMeta
import com.under9.android.lib.util.file.SourceFileController
import com.under9.android.lib.util.BitmapUtil
import timber.log.Timber

import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.io.IOException

/**
 * Created by raymond on 9/5/2017.
 */

class ImageProcessor(context: Context,
                     sfc: SourceFileController,
                     saveMediaCallback: SaveMediaCallback,
                     mediaValidator: BaseMediaValidator, private val shouldCompress: Boolean = true
) : BaseProcessor(context, sfc, saveMediaCallback, mediaValidator) {

    override fun getMediaMeta(f: File): MediaMeta {
        val opts = BitmapUtil.getBitmapBound(f.absolutePath)
        var width = 0
        var height = 0

        if (opts != null) {
            width = opts.outWidth
            height = opts.outHeight
        }

        return MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_IMAGE)
                .filePath(f.absolutePath)
                .fileSize(f.length())
                .widthAndHeight(width, height)
                .build()
    }

    override fun saveMedia(parcelFileDescriptor: ParcelFileDescriptor, contentUri: Uri, tmpFileLocation: String): MediaMeta {
        val fos = FileOutputStream(tmpFileLocation)
        val fis = FileInputStream(parcelFileDescriptor.fileDescriptor)

        val opts = BitmapUtil.getBitmapBound(context, contentUri)
        var width = 0
        var height = 0
        if (opts != null) {
            width = opts.outWidth
            height = opts.outHeight
        }

        val bitmap = BitmapUtil.smartDecodeBitmap(context, contentUri, Constant.IMAGE_SIDE_LIMIT)
        if (shouldCompress) {
            bitmap!!.compress(Bitmap.CompressFormat.JPEG, Constant.JPEG_QUALITY, fos)
        } else {
            bitmap!!.compress(Bitmap.CompressFormat.JPEG, Constant.ORIGINAL_QUALITY, fos)
        }
        fos.close()

        return MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_IMAGE)
                .filePath(tmpFileLocation)
                .fileSize(fis.channel.size())
                .widthAndHeight(width, height)
                .build()
    }

    @Throws(IOException::class, NullPointerException::class)
    override fun saveMedia(mediaMeta: MediaMeta, tmpFileLocation: String) {
        val fos = FileOutputStream(tmpFileLocation)
        val path = mediaMeta.filePath.replace("%20".toRegex(), " ")
        val bitmap = BitmapUtil.smartDecodeBitmap(path, Constant.IMAGE_SIDE_LIMIT)
        Timber.d("path=$path")
        if (shouldCompress) {
            bitmap!!.compress(Bitmap.CompressFormat.JPEG, Constant.JPEG_QUALITY, fos)
        } else {
            bitmap!!.compress(Bitmap.CompressFormat.JPEG, Constant.ORIGINAL_QUALITY, fos)
        }
        fos.close()
    }
}
