package com.ninegag.android.library.upload.media.validator

import com.ninegag.android.library.upload.model.MediaMeta

/**
 * Created by raymond on 9/5/2017.
 */

abstract class BaseMediaValidator(
        val baseValidatorConfig: BaseValidatorConfig, val mediaValidatorCallback: MediaValidatorCallback
) : MediaValidator {

    interface MediaValidatorCallback {
        fun onFileSizeLimitExceeded(fileSize: Long, config: BaseValidatorConfig)
        fun onFileNotFound()
        fun onMediaDimenTooSmall(width: Int, height: Int, config: BaseValidatorConfig)
        fun onMediaDimenTooLarge(width: Int, height: Int, config: BaseValidatorConfig)
        fun onMediaDimenTooLong(width: Int, height: Int, config: BaseValidatorConfig)
        fun onMediaPixelLimitExceeded(width: Int, height: Int, config: BaseValidatorConfig)
    }

    override fun validateFileSize(mediaMeta: MediaMeta): Boolean {
        if (mediaMeta.fileSize > baseValidatorConfig.fileSizeLimit) {
            mediaValidatorCallback.onFileSizeLimitExceeded(baseValidatorConfig.fileSizeLimit, baseValidatorConfig)
            return false
        }

        if (mediaMeta.fileSize <= 0) {
            mediaValidatorCallback.onFileNotFound()
            return false
        }

        return true
    }
}
