package com.ninegag.android.library.upload.media.validator

import com.ninegag.android.library.upload.model.MediaMeta

import com.ninegag.android.library.upload.Constant.IMAGE_SIDE_LIMIT

/**
 * Base image validator class, to validate all sorts of images
 * Created by raymond on 9/5/2017.
 */

class ImageValidator(
        config: BaseValidatorConfig, callback: MediaValidatorCallback
) : BaseMediaValidator(config, callback) {

    override fun validateFileDimension(mediaMeta: MediaMeta): Boolean {
        val config = baseValidatorConfig
        val callback = mediaValidatorCallback

        if (mediaMeta.width < config.minWidth || mediaMeta.height < config.minHeight) {
            callback.onMediaDimenTooSmall(mediaMeta.width, mediaMeta.height, config)
            return false
        }

        if (mediaMeta.width > config.maxWidth || mediaMeta.height > config.maxHeight) {
            callback.onMediaDimenTooLarge(mediaMeta.width, mediaMeta.height, config)
            return false
        }

        var scaledW = mediaMeta.width.toLong()
        var scaledH = mediaMeta.height.toLong()
        // Support HD image for comment and Post
        if (mediaMeta.width > IMAGE_SIDE_LIMIT) {
            scaledW = IMAGE_SIDE_LIMIT.toLong()
            scaledH = scaledW * mediaMeta.height / mediaMeta.height
        }

        if (scaledW * scaledH / config.displayStandardWidth > config.heightDisplayMax) {
            callback.onMediaDimenTooLong(mediaMeta.width, mediaMeta.height, config)
            return false
        }

        if (scaledW * scaledH > config.pixelLimit) {
            callback.onMediaPixelLimitExceeded(mediaMeta.width, mediaMeta.height, config)
            return false
        }

        return true
    }
}
