package com.under9.android.comments.adapter

import android.view.View
import android.widget.CompoundButton
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.controller.MetricsController
import com.under9.android.comments.model.CommentItem
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.comments.ui.renderer.BubbleVoteInfoRenderer
import com.under9.android.comments.ui.renderer.MentionedUserClickSpan
import com.under9.android.comments.ui.renderer.UrlClickSpan
import com.under9.android.comments.ui.renderer.VoteInfoRenderer
import com.under9.android.comments.ui.widget.BaseCommentItemView
import com.under9.android.comments.ui.widget.BubbleCommentView
import com.under9.android.commentsystem.R
import com.under9.android.lib.blitz.adapter.BlitzSubAdapter
import com.under9.android.lib.widget.uiv.v3.UniversalImageView
import com.under9.android.lib.widget.uiv.v3.adapter.UIVAdapter
import timber.log.Timber

class CommentItemClickListenerV2(
    private val commentItemActionHandler: CommentItemAction
): View.OnClickListener, CompoundButton.OnCheckedChangeListener,
        UrlClickSpan.OnClickListener, MentionedUserClickSpan.OnClickListener,
        UniversalImageView.OnClickListener, UniversalImageView.OnLongClickListener {

    /**
     * Dirty hack here, we don't let notifyDataXXX to rebinding the element and update the count,
     * this is because MP4 playback will be stopped.
     *
     * We call it and update UI after event is handled immediately here.
     */
    private val voteInfoRenderer by lazy(LazyThreadSafetyMode.NONE) {
        VoteInfoRenderer(null, null)
    }
    private val bubbleVoteInfoRenderer by lazy(LazyThreadSafetyMode.NONE) {
        BubbleVoteInfoRenderer(null, null)
    }

    override fun onClick(view: View, adapter: UIVAdapter, uiv: UniversalImageView) {
        commentItemActionHandler.onImageClick(view, adapter, uiv)
    }

    override fun onLongClick(view: View, adapter: UIVAdapter, uiv: UniversalImageView) {
        commentItemActionHandler.onImageLongClick(view, adapter, uiv)
    }

    // MentionedUserClickSpan.OnClickListener
    override fun onClick(username: String?, accountId: String?) {
        if (username == null || accountId == null) return
        commentItemActionHandler.onMentionedUserClick(username, accountId)
    }

    // UrlClickSpan.OnClickListener
    override fun onClick2(commentWrapper: CommentItemWrapperInterface, url: String) {
        commentItemActionHandler.onItemUrl(commentWrapper, url)
    }

    override fun onCheckedChanged(v: CompoundButton?, isChecked: Boolean) {
        Timber.d("isChecked = $v")
    }

    override fun onClick(v: View?) {
        if (v == null) return

        val id = v.id
        val context = v.context
        val comment: CommentItemWrapperInterface = v.tag as CommentItemWrapperInterface
        val position: Int = (v.getTag(R.id.commentItemViewHolder) as BlitzSubAdapter.ViewHolder).subAdapterPosition
        val commentItemView: BaseCommentItemView = (v.getTag(R.id.commentItemViewHolder) as RecyclerView.ViewHolder).itemView as BaseCommentItemView

        when (id) {
            R.id.replyBtnV4 -> {
                val prefill = "@${comment.getUser().getDisplayName()} "
                commentItemActionHandler.onItemReply(position, comment, prefill, null)
            }

            R.id.loadMoreContainer -> {
                if (comment.getLevel() > 1) return

                commentItemActionHandler.onLoadMoreContainerClick(position, comment)
            }

            R.id.loadPrevTxt, R.id.loadPrevContainer -> {
                commentItemActionHandler.onLoadPrevContainerClick(position, comment)
            }

            R.id.moreBtn -> {
                MetricsController.logUserActions(MetricsController.USER_ACTIONS_OPTIONS)
                commentItemActionHandler.onItemMore(position, comment, comment.getUser().getName())
            }

            R.id.userName, R.id.avatar -> {
                val authorName = v.getTag(R.id.username) as String?
                authorName?.let { commentItemActionHandler.onAuthorNameClick(authorName, comment) }
            }

            R.id.textBubbleBackground -> {
                if (commentItemView is BubbleCommentView) {
//                    if (commentItemView.bubbleMode == CommentListItemAdapter.MODE_BUBBLE_COMMENT_HIDE_ACTION) {
                        commentItemView.hideShowInteractionBar()
//                    } else {
//                        if (comment.getLevel() > 1) return
//                        commentItemActionHandler.onLoadMoreContainerClick(position, comment)
//                    }
                }
            }

            R.id.content, R.id.cs_rootView -> {
                if (v is BubbleCommentView) {
//                    if (v.bubbleMode == CommentListItemAdapter.MODE_BUBBLE_COMMENT_HIDE_ACTION) {
                        v.hideShowInteractionBar()
//                    } else {
//                        if (comment.getLevel() > 1) return
//                        commentItemActionHandler.onLoadMoreContainerClick(position, comment)
//                    }
                } else {
                    if (comment.getLevel() > 1) return
                    commentItemActionHandler.onLoadMoreContainerClick(position, comment)
                }
            }

            R.id.proBadge -> {
                commentItemActionHandler.onItemProBadge(comment)
            }

            R.id.refresh -> {

            }

            R.id.likeBtn,
            R.id.likeBtnMask,
            R.id.circleLikeBtn,
            R.id.circleLikeContainer -> {
                if (commentItemView is BubbleCommentView) {
                    // if user touch the circle like container instead of the circle upvote checkbox
                    val upvoteCheckBox = if (id == R.id.circleLikeContainer) {
                        commentItemView.findViewById(R.id.circleLikeBtn)
                    } else {
                        v
                    }
                    if (comment.likeStatus == CommentItem.LIKE_STATUS_LIKED) {
                        commentItemActionHandler.onItemUnlike(position, comment)
                        (upvoteCheckBox as CompoundButton).isChecked = false
                        bubbleVoteInfoRenderer.pureBind(comment, commentItemView)
                    } else {
                        val result = commentItemActionHandler.onItemLike(position, comment)
                        if (result) {
                            commentItemView.runAnimation(v, comment.likeStatus)
                            bubbleVoteInfoRenderer.pureBind(comment, commentItemView)
                        }
                        (upvoteCheckBox as CompoundButton).isChecked = result
                    }
                } else {
                    if (comment.likeStatus == CommentItem.LIKE_STATUS_LIKED) {
                        commentItemActionHandler.onItemUnlike(position, comment)
                        (v as CompoundButton).isChecked = false
                        voteInfoRenderer.pureBind(comment, commentItemView)
                    } else {
                        val result = commentItemActionHandler.onItemLike(position, comment)
                        if (result) {
                            commentItemView.runAnimation(v, comment.likeStatus)
                            voteInfoRenderer.pureBind(comment, commentItemView)
                        }
                        (v as CompoundButton).isChecked = result
                    }
                }
            }
            R.id.dislikeBtn,
            R.id.dislikeBtnMask -> {
                if (commentItemView is BubbleCommentView) {

                    if (comment.likeStatus == CommentItem.LIKE_STATUS_DISLIKED) {
                        commentItemActionHandler.onItemUnlike(position, comment)
                        (v as CompoundButton).isChecked = false
                        bubbleVoteInfoRenderer.pureBind(comment, commentItemView)
                    } else {
                        val result = commentItemActionHandler.onItemDislike(position, comment)
                        if (result) {
                            commentItemView.runAnimation(v, comment.likeStatus)
                            bubbleVoteInfoRenderer.pureBind(comment, commentItemView)
                        }
                        (v as CompoundButton).isChecked = result
                    }
                } else {
                    if (comment.likeStatus == CommentItem.LIKE_STATUS_DISLIKED) {
                        commentItemActionHandler.onItemUnlike(position, comment)
                        (v as CompoundButton).isChecked = false
                        voteInfoRenderer.pureBind(comment, commentItemView)
                    } else {
                        val result = commentItemActionHandler.onItemDislike(position, comment)
                        if (result) {
                            commentItemView.runAnimation(v, comment.likeStatus)
                            voteInfoRenderer.pureBind(comment, commentItemView)
                        }
                        (v as CompoundButton).isChecked = result
                    }
                }
            }
            R.id.textBubbleBackground -> {
                if (commentItemView is BubbleCommentView) {
//                    if (commentItemView.bubbleMode == CommentListItemAdapter.MODE_BUBBLE_COMMENT_HIDE_ACTION) {
                        commentItemView.hideShowInteractionBar()
//                    } else {
//                        if (comment.getLevel() > 1) return
//                        commentItemActionHandler.onLoadMoreContainerClick(position, comment)
//                    }
                }
            }

            else -> {}
        }
    }
}