package com.under9.android.comments.adapter

import android.os.Bundle
import android.util.ArrayMap
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.RepositoryManager
import com.under9.android.comments.data.query.comment.CommentListQueryResult2
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.comments.model.wrapper.CommentListItemWrapper
import com.under9.android.comments.ui.renderer.BubbleCommentRendererDelegate
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2
import com.under9.android.comments.ui.widget.BubbleCommentView
import com.under9.android.comments.ui.widget.CommentItemView
import com.under9.android.lib.blitz.adapter.BlitzItemAdapter

class CommentListItemAdapter(
        commentListItemWrapper: CommentListItemWrapper,
        renderOptions: Bundle?,
        commentItemActionHandler: CommentItemAction,
        private val viewMode: Int = MODE_COMMENT,
        userAccentColorMap: ArrayMap<String, Int>
) : BlitzItemAdapter<CommentItemWrapperInterface>(commentListItemWrapper.list) {

    companion object {
        const val MODE_COMMENT = 1
        const val MODE_BUBBLE_COMMENT_HIDE_ACTION = 2
        const val MODE_BUBBLE_COMMENT_QUICK_ACTION = 3

    }

    internal class VH(view: View): ViewHolder(view)

    /**
     * @see CommentListQueryResult2.level
     */
    var level: Int = 1
    var selectedCommentId: String? = null
    set(value) {
        field = value
    }

    private val themeAttr = CommentItemThemeAttr()

    private var commentItemRendererDelegate = when (viewMode) {
        MODE_COMMENT -> CommentItemRendererDelegateV2(
                CommentSystem.getInstance().config.domainMapperModule,
                RepositoryManager.appInfoRepository,
                renderOptions,
                commentItemActionHandler,
                userAccentColorMap
        )
        MODE_BUBBLE_COMMENT_HIDE_ACTION, MODE_BUBBLE_COMMENT_QUICK_ACTION -> BubbleCommentRendererDelegate(CommentSystem.getInstance().config.domainMapperModule,
                RepositoryManager.appInfoRepository,
                renderOptions,
                commentItemActionHandler,
                userAccentColorMap)
        else -> throw IllegalArgumentException("no this comment view mode")
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val v = when (viewMode) {
            MODE_COMMENT -> CommentItemView(parent.context).also { it.initView() }
            MODE_BUBBLE_COMMENT_HIDE_ACTION -> BubbleCommentView(parent.context, MODE_BUBBLE_COMMENT_HIDE_ACTION).also { it.initView() }
            MODE_BUBBLE_COMMENT_QUICK_ACTION -> BubbleCommentView(parent.context, MODE_BUBBLE_COMMENT_QUICK_ACTION).also { it.initView() }
            else -> throw IllegalArgumentException("no this comment view mode")
        }
        v.layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        v.commentItemThemeAttr = themeAttr
        themeAttr.mayInitThemeAttr(parent.context)
        return VH(v)
    }

    override fun onBindViewHolder(vh: ViewHolder, i: Int) {
        super.onBindViewHolder(vh, i)
        commentItemRendererDelegate.bind(level, getItem(i), themeAttr, vh, i)
    }

    override fun getItemCount(): Int {
        return super.getItemCount()
    }

    fun refreshRenderOptions(renderOptions: Bundle?) {
        commentItemRendererDelegate.refreshRenderOptions(renderOptions)
    }
}