package com.under9.android.comments.controller

import com.under9.android.comments.CommentSystem
import java.util.*

object MetricsController {
    private const val TAG = "MetricsController"
    fun logCommentNullProblem(msg: String) {
        log(RLOG_KEY_COMMENT_NULL_PROBLEM, RLOG_CATEGORY_COMMENT, msg)
    }

    @JvmStatic
    fun logCommentAddProblem(msg: String?) {
        log(RLOG_KEY_COMMENT_ADD_PROBLEM, RLOG_CATEGORY_COMMENT, msg)
    }

    @JvmStatic
    fun logApiUnexpectedDataType(message: String?, objJSON: String) {
        log(RLOG_KEY_API_UNEXPECTED_DATA_TYPE, RLOG_CATEGORY_API_ERRORS, message + "\n" + objJSON)
    }

    @JvmStatic
    fun logApiUnexpectedDataTypeInvalidJSONObject(objJSON: String?) {
        log(RLOG_KEY_API_UNEXPECTED_DATA_TYPE, RLOG_CATEGORY_API_ERRORS, "Not a valid JSON Object\n$objJSON")
    }

    @JvmStatic
    fun logApiNoQuota(msg: String?) {
        log(RLOG_KEY_NO_QUOTA, RLOG_CATEGORY_API_ERRORS, msg)
    }

    fun logCommentMediaUnexpectedNull(commentId: String, isPending: Boolean) {
        log(RLOG_COMMENT_MEDIA_PROBLEM, RLOG_CATEGORY_COMMENT, "Comment Media Null id is " + commentId + "isPending " + isPending)
    }

    const val USER_ACTIONS_COMPOSE = "compose"
    const val USER_ACTIONS_COMPOSE_FAILED_OTHERS = "compose-failed.others"
    const val USER_ACTIONS_COMPOSE_FAILED_QUOTA_EXCEEDED = "compose-failed.quota-exceeded"
    const val USER_ACTIONS_COMPOSE_FAILED_USER_NOT_FOUND = "compose-failed.user-not-found"
    const val USER_ACTIONS_DELETE = "delete"
    const val USER_ACTIONS_DOWNVOTE = "downvote"
    const val USER_ACTIONS_INFINITE_SCROLLING = "infinite-scrolling"
    const val USER_ACTIONS_MENU = "menu"
    const val USER_ACTIONS_OPTIONS = "options"
    const val USER_ACTIONS_REPLY = "reply"
    const val USER_ACTIONS_REPORT = "report"
    const val USER_ACTIONS_SUB_LOAD_MORE = "sub-load-more"
    const val USER_ACTIONS_UPVOTE = "upvote"
    const val USER_ACTIONS_HIDE = "hide"
    @JvmStatic
    @Deprecated("Use other logging call")
    fun logUserActions(action: String) {
        val suffix = ".comment_system.$action"
        logCount(RLOG_CATEGORY_USER_ACTIONS + suffix, 1)
    }

    @Deprecated("Use other logging call")
    private fun logCount(key: String, count: Int) {
        val logger = CommentSystem.getInstance().logger
        logger?.logCount(key, count)
    }

    @JvmStatic
    fun log(key: String, category: String, message: String?) {
        val logger = CommentSystem.getInstance().logger
        logger?.log(key, category, message)
    }

    @JvmStatic fun log(key: String, category: String, count: Int, message: String?) {
        val logger = CommentSystem.getInstance().logger
        logger?.log(key, count, category, message)
    }

    private fun log(key: String, category: String, message: String?, totalTime: Long) {
        val logger = CommentSystem.getInstance().logger
        logger?.log(key, category, message, totalTime)
    }

    private fun startTimedLog(key: String, category: String, message: String?) {
        val logger = CommentSystem.getInstance().logger
        logger?.startTimedLog(key, category, message)
    }

    private fun endTimedLog(key: String) {
        val logger = CommentSystem.getInstance().logger
        logger?.endTimedLog(key)
    }

    private fun logBreadcrumb(key: String, jsonMessage: String) {
        val logger = CommentSystem.getInstance().logger
        logger?.logBreadcrumb(key, jsonMessage)
    }

    private fun logBreadcrumb(key: String, jsonMessageMap: HashMap<String, Any>) {
        val logger = CommentSystem.getInstance().logger
        logger?.logBreadcrumb(key, jsonMessageMap)
    }

    const val RLOG_CATEGORY_COMMENT = "COMMENT"
    const val RLOG_KEY_REMOTE_REFRESH_COMMENT_NULL_PROBLEM = "PROBLEM_REMOTE_REFRESH_COMMENT_NULL"
    const val RLOG_KEY_COMMENT_NULL_PROBLEM = "COMMENT_NULL_PROBLEM"
    const val RLOG_KEY_COMMENT_ADD_PROBLEM = "COMMENT_ADD_PROBLEM"
    const val RLOG_CATEGORY_API_ERRORS = "COMMENT_API_ERRORS"
    const val RLOG_KEY_API_UNEXPECTED_DATA_TYPE = "UNEXPECTED_DATA_TYPE"
    const val RLOG_KEY_NO_QUOTA = "NO_QUOTA"
    const val RLOG_KEY_USER_NOT_FOUND = "USER_NOT_FOUND"
    const val RLOG_KEY_OTHER_ERROR = "OTHER_ERROR"
    const val RLOG_CATEGORY_USER_ACTIONS = "USER_ACTIONS"
    const val RLOG_COMMENT_MEDIA_PROBLEM = "COMMENT_MEDIA_PROBLEM"

    object FirebaseEvent {
        const val COMMENT_USER_INFO_VISIBLE_REMOTE = "comment_user_info_visible_remote"
    }
}