package com.under9.android.comments.data.processor

import android.util.ArrayMap
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.comment.CommentListQueryParam2
import com.under9.android.comments.data.query.comment.CommentListQueryResult2
import com.under9.android.comments.data.repository.LocalCommentListRepositoryInterface
import com.under9.android.comments.data.repository.LocalUserRepositoryInterface
import com.under9.android.comments.model.CommentItem
import com.under9.android.comments.model.api.ApiCommentList
import com.under9.android.comments.model.api.ApiUser
import io.reactivex.Flowable

class CommentListPrevProcessor(
        dataController: DataController,
        private val queryParam: CommentListQueryParam2,
        private val localUserRepository: LocalUserRepositoryInterface,
        private val localCommentListRepository: LocalCommentListRepositoryInterface
): BaseApiResponseProcessor<ApiCommentList>(dataController) {

    private val listKey = queryParam.listKey
    private val localNextOffset = queryParam.localNextOffset
    private val direction = queryParam.direction
    private val isRefresh = queryParam.isRefresh

    override fun process(apiResponse: ApiCommentList): Flowable<CommentListQueryResult2> {
        val apiComments = apiResponse.payload.comments
        val opUserId = apiResponse.payload.opUserId
        val level = apiResponse.payload.level

        // 1. Get all local users
        //      1.1 Create new one if not exists
        //      1.2 Update existing one
        // 2. Get all local comments
        //      2.1 Create new one if not exists
        //      2.2 Update existing one

        // Insert new users, update existing users
        val apiUsers = apiComments.map { it.user }
        val localUserIdMapping = localUserRepository.insertOrUpdateFromApiCommentUsers(apiUsers)

        localCommentListRepository.insertOrUpdateFromApiComments(queryParam,
                level, apiComments,
                localUserIdMapping)

        val items = localCommentListRepository.getNextListItems(listKey, level - 1, apiComments.size)
        val listMeta = localCommentListRepository.getListMeta(listKey, direction)

        return Flowable.just(CommentListQueryResult2(
                listMeta[CommentListQueryParam2.KEY_PREV],
                listMeta[CommentListQueryParam2.KEY_NEXT], opUserId, level, items))
    }
}