package com.under9.android.comments.data.processor

import androidx.collection.ArrayMap
import com.google.gson.reflect.TypeToken
import com.under9.android.comments.AppOptions
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.user.UserStatusQueryResult
import com.under9.android.comments.model.api.ApiUserStatus
import com.under9.android.comments.GsonFactory
import com.under9.android.comments.data.repository.LocalCommentListRepository
import com.under9.android.comments.data.repository.LocalCommentListRepositoryInterface
import io.reactivex.Flowable

class UserStatusProcessor(
        dataController: DataController,
        localCommentListRepository: LocalCommentListRepositoryInterface
): BaseApiResponseProcessor<ApiUserStatus>(dataController) {

    override fun process(apiResponse: ApiUserStatus): Flowable<UserStatusQueryResult> {
        apiResponse.payload?.let { payload ->
            payload.user?.takeIf { it.userId != null }?.let { apiUser ->
                val user = dataController.obtainUser(apiUser)
                dataController.setStringData(AppOptions.FIELD_LOGGED_IN_USER, user.userId)
            }

            payload.quota?.let {
                dataController.setCommentQuota(it.count, it.wait)
            }

            payload.likeMapping?.let {
                val likeMapping = GsonFactory.getGson().toJson(it)
                dataController.setStringData(AppOptions.FIELD_LIKE_MAPPING, likeMapping)
            }

            payload.reportedPostUrls?.let {
                val reportedPostUrls = GsonFactory.getGson().toJson(it)
                dataController.setStringData(AppOptions.FIELD_REPORTED_POST_URLS, reportedPostUrls)
            }

            payload.commentedPostUrls?.let {
                val commentedPostUrls = GsonFactory.getGson().toJson(it)
                dataController.setStringData(AppOptions.FIELD_COMMENTED_POST_LIKE_URLS, commentedPostUrls)
            }

            dataController.setLongData(AppOptions.FIELD_LAST_USER_STATUS_QUERY_TIME, System.currentTimeMillis())

            return Flowable.just(UserStatusQueryResult(payload.likeMapping, payload.commentedPostUrls, payload.reportedPostUrls))
        }

        val gson = GsonFactory.getGson()
        val mapType = object: TypeToken<ArrayMap<String, Int>>() {}.type

        val cachedLikeMapping = gson.fromJson<Map<String, Int>>(dataController.getStringData(AppOptions.FIELD_LIKE_MAPPING), mapType)
        val cachedReportedPostUrls = gson.fromJson<Map<String, Int>>(dataController.getStringData(AppOptions.FIELD_REPORTED_POST_URLS), mapType)
        val cachedCommentedPostUrls = gson.fromJson<Map<String, Int>>(dataController.getStringData(AppOptions.FIELD_COMMENTED_POST_LIKE_URLS), mapType)

        return Flowable.just(UserStatusQueryResult(cachedLikeMapping, cachedCommentedPostUrls, cachedReportedPostUrls, true))
    }

}