package com.under9.android.comments.data.repository

import com.under9.android.comments.data.query.comment.CommentListQueryParam2
import com.under9.android.comments.model.CommentItem
import com.under9.android.comments.model.CommentList
import com.under9.android.comments.model.CommentListItem
import com.under9.android.comments.model.User
import com.under9.android.comments.model.api.ApiComment

interface LocalCommentListRepositoryInterface {
    /**
     * Get current list's listKey
     */
    fun obtainListByListKey(listKey: String): CommentList

    /**
     * Update listKey's values
     *
     * @param listKey
     * @param prevCursor
     * @param nextCursor
     */
    fun updateListByListKey(listKey: String, prevCursor: String?,
                            nextCursor: String?, opUserId: String, level: Int, direction: Int, lastRefreshTs: Long)

    /**
     * @param listKey
     */
    fun getCommentListItem(listKey: String): List<CommentListItem>

    fun getCommentItemById(commentId: String): CommentItem?

    /**
     * Get comment by local DB ID
     * @param dbId
     */
    fun getCommentItemByDbId(dbId: Long): CommentItem?

    /**
     * @param ids Comment ids
     */
    fun getCommentItemsByIds(ids: Collection<String>): List<CommentItem>

    /**
     * @param ids Comment ids
     */
    fun getCommentListItemsByIds(ids: Collection<String>): List<CommentListItem>

    fun getLatestComment(listKey: String): CommentItem?

    /**
     *
     */
    fun replaceCommentItem(url: String, item: ApiComment, commentUser: User): CommentItem

    /**
     * Replace a pending comment with a the one obtained from API
     */
    fun replacePendingCommentItem(url: String, id: Long, item: ApiComment, commentUser: User): CommentItem?


    /**
     * Insert or [CommentListItem], [CommentItem] by batches
     *
     * @param insertCommentItemBatches list of [CommentItem]
     * @param updateCommentItemBatches list of [CommentListItem]
     * @param insertCommentListItemBatches list of [CommentListItem]
     * @param updateCommentListItemBatches list of [CommentListItem]
     */
    fun insertOrUpdateCommentItemBatches(insertCommentItemBatches: List<CommentItem>,
                                         updateCommentItemBatches: List<CommentItem>,
                                         insertCommentListItemBatches: List<CommentListItem>,
                                         updateCommentListItemBatches: List<CommentListItem>)

    /**
     * @param queryParam
     * @param apiCommentListLevel Level returned from [ApiComment.level]
     * @param apiComments list of [ApiComment]
     * @param userMapping mapping of user id (from API) to local [User] object
     */
    fun insertOrUpdateFromApiComments(queryParam: CommentListQueryParam2,
                                      apiCommentListLevel: Int,
                                      apiComments: List<ApiComment>,
                                      userMapping: Map<String, User>)

    /**
     * Find out the latest insert ordering number
     */
    fun getMaxLocalInsertOrder(listKey: String): Long

    /**
     * Update the inserted order
     */
    fun updateLocalInsertOrder(listKey: String, startingOrder: Int, prependSize: Int)

    fun getNextListItems(listKey: String, offset: Int, limit: Int): List<CommentListItem>

    fun replaceParent(listKey: String, url: String, item: ApiComment, commentUser: User)

    fun getListMeta(listKey: String, direction: Int): Map<String, String?>

    fun clearList(listKey: String)

    fun updateCommentItems(commentItems: List<CommentItem>)

    fun createPendingCommentItem(url: String, listKey: String, level: Int,
                                 content: String, parent: String?, sourceMetaJson: String?, sourceMetaHash: String?): CommentListItem

    fun getCommentListItemByDbId(id: Long): CommentListItem?
    
    fun getCommentListItemByCommentDbId(id: Long): List<CommentListItem>

    fun clearPendingItemByDbId(id: Long)

    fun clear()

    fun deleteComment(commentId: String, commentUrl: String, reportType: Int)
}