package com.under9.android.comments.data.repository

import androidx.collection.ArrayMap
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.model.ModelFactory
import com.under9.android.comments.model.User
import com.under9.android.comments.model.UserDao
import com.under9.android.comments.model.api.ApiUser

class LocalUserRepository(
        private val dataController: DataController
) : LocalUserRepositoryInterface {

    /**
     * @param apiUser
     */
    override fun obtainUser(apiUser: ApiUser): User? {
        var user = getUserByUserId(apiUser.userId)
        if (user == null) {
            user = ModelFactory.createUser(apiUser, dataController.appId, null)
            dataController.userDao.insert(user)
        } else {
            user = ModelFactory.createUser(apiUser, dataController.appId, user)
            dataController.userDao.update(user)
        }
        return user
    }

    override fun getUserByUserId(userId: String): User? {
        val list = dataController.userDao.queryBuilder()
                .where(UserDao.Properties.UserId.eq(userId))
                .build()
                .forCurrentThread()
                .list()

        return if (list.isEmpty()) null else list[0]
    }

    override fun getUsersByUserIds(userIds: List<String>): List<User> {
        return dataController.userDao.queryBuilder()
                .where(UserDao.Properties.UserId.`in`(userIds))
                .list()
    }

    override fun insertOrUpdateUserBatches(insertUserBatches: List<User>, updateUserBatches: List<User>) {
        dataController.session.runInTx {
            dataController.userDao.insertInTx(insertUserBatches)
            dataController.userDao.updateInTx(updateUserBatches)
        }
    }

    override fun insertOrUpdateFromApiCommentUsers(apiUsers: List<ApiUser>): Map<String, User> {
        val apiUserIds = apiUsers.map { it.userId }
        val localUsers = getUsersByUserIds(apiUserIds)
        val localUserMapping = ArrayMap<String, User>()
        localUsers.forEach { localUserMapping[it.userId] = it }

        val newUsers = arrayListOf<User>()
        val oldUsers = arrayListOf<User>()
        val mergedUserMapping = ArrayMap<String, User>()
        for (apiUser in apiUsers) {
            val existingLocalUser = localUserMapping[apiUser.userId]
            val updatedUser = if (localUserMapping.containsKey(apiUser.userId)) {
                val oldUser = ModelFactory.createUser(apiUser, CommentSystem.getInstance().appId, existingLocalUser)
                oldUsers.add(oldUser)
                oldUser
            } else {
                val newUser = ModelFactory.createUser(apiUser, CommentSystem.getInstance().appId, null)
                newUsers.add(newUser)
                newUser
            }

            mergedUserMapping[apiUser.userId] = updatedUser
        }
        insertOrUpdateUserBatches(newUsers, oldUsers)
        return mergedUserMapping
    }
}