package com.under9.android.comments.model

import com.under9.android.comments.GsonFactory
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.model.api.ApiComment
import com.under9.android.comments.model.api.ApiUser
import com.under9.android.lib.util.GsonUtil.toJson

object ModelFactory {
    @JvmStatic
    fun createUser(apiUser: ApiUser, appId: String?, user: User?): User {
        var user = user
        if (user == null) {
            user = User()
        }
        user.avatarUrl = apiUser.avatarUrl
        user.displayName = apiUser.displayName
        user.isActivePro = apiUser.isActivePro
        user.isActiveProPlus = apiUser.isActiveProPlus
        user.userId = apiUser.userId
        user.accountId = apiUser.accountId
        if (apiUser.profileUrls == null) {
            user.profileUrls = ""
            user.profileUrl = ""
        } else {
            user.profileUrls = toJson(apiUser.profileUrls)
            user.profileUrl = apiUser.profileUrls[appId]
        }
        user.location = apiUser.location
        user.country = apiUser.country
        user.emojiStatus = apiUser.emojiStatus
        user.activeTs = apiUser.activeTs
        if (apiUser.userPrefs != null) {
            user.hideProBadge = apiUser.userPrefs!!.hideProBadge
            user.hideActiveTs = apiUser.userPrefs!!.hideActiveTs
            user.userBackgroundColor = apiUser.userPrefs!!.backgroundColor
            user.userAccentColor = apiUser.userPrefs!!.accentColor
        }
        return user
    }

    fun createCommentItem(url: String, item: ApiComment, commentUser: User): CommentItem {
        val commentItem = CommentItem()
        applyCommentItem(url, commentItem, item, commentUser)
        return commentItem
    }

    fun updateCommentItem(url: String,
                          item: ApiComment,
                          commentItem: CommentItem, commentUser: User): CommentItem {
        return applyCommentItem(url, commentItem, item, commentUser)
    }

    private fun applyCommentItem(url: String, commentItem: CommentItem, item: ApiComment, commentUser: User): CommentItem {
        commentItem.apply {
            this.url = url
            commentId = item.commentId
            threadId = item.threadId
            text = item.text
            timestamp = item.timestamp
            mentionMapping = GsonFactory.getGson().toJson(item.mentionMapping)
            type = item.type
            permalink = item.permalink
            level = item.level
            isVoteMasked = item.isVoteMasked
            mediaText = item.mediaText
            likeCount = item.likeCount
            dislikeCount = item.dislikeCount
            childrenTotal = item.childrenTotal
            childrenUrl = item.childrenUrl
            parent = item.parent
            media = if (item.media != null) GsonFactory.getGson().toJson(item.media) else null
            user = commentUser
        }
        return commentItem
    }


    fun createOrUpdateCommentListItem(startingOrder: Long,
                                      listKey: String,
                                      oldCommentItem: CommentItem,
                                      oldCommentListItem: CommentListItem?): CommentListItem {

        val listItem = oldCommentListItem ?: CommentListItem()

        listItem.apply {
            this.listKey = listKey
            this.commentId = oldCommentItem.commentId
            this.commentItem = oldCommentItem
            this.localInsertOrder = startingOrder
            this.status = if (oldCommentListItem == null) CommentItem.STATUS_NORMAL else listItem.status
        }

        return listItem
    }
}