package com.under9.android.comments.model.wrapper

import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.lib.widget.media.overlayv3.IOverlayable
import com.under9.android.lib.widget.media.wrapper.IMediaWrapperV2

object CommentTransformer {


    fun transform2(commentWrapper: CommentItemWrapperInterface, isEnableHD: Boolean): IMediaWrapperV2? {
        if (commentWrapper.getFirstMedia() == null) return null

        val media = commentWrapper.getFirstMedia()!!.imageMetaByType

        with(media) {
            if (media?.type == null) return null
            val hasHD = isEnableHD && media.imageXLarge != null

            when (type) {
                CommentConstant.MEDIA_TYPE_STATIC -> {

                    val imageUrl = if (!hasHD) {
                        media.image.getImageUrl()
                    } else {
                        media.imageXLarge.getImageUrl()
                    }

                    val width = if (!hasHD) {
                        media.image.width
                    } else {
                        media.imageXLarge.width
                    }

                    val height = if (!hasHD) {
                        media.image.height
                    } else {
                        media.imageXLarge.height
                    }

                    return CommentTransformWrapper(
                            mediaHeight = height,
                            mediaWidth = width,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            otherVideo = media.animated == null && media.video != null,
                            commentType = if (type == CommentConstant.MEDIA_TYPE_ANIMATED) IMediaWrapperV2.MEDIA_TYPE_ANIMATED else IMediaWrapperV2.MEDIA_TYPE_STATIC,
                            mediaImageUrl = imageUrl,
                            mediaVideoUrl = null,
                            isEnabledHD = hasHD
                    )
                }

                CommentConstant.MEDIA_TYPE_ANIMATED -> {
                    return CommentTransformWrapper(
                            mediaHeight = image!!.height,
                            mediaWidth = image!!.width,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            otherVideo = media.animated == null && media.video != null,
                            commentType = if (type == CommentConstant.MEDIA_TYPE_ANIMATED) IMediaWrapperV2.MEDIA_TYPE_ANIMATED else IMediaWrapperV2.MEDIA_TYPE_STATIC,
                            mediaImageUrl = null,
                            mediaVideoUrl = video.url,
                            isEnabledHD = hasHD)
                }
                // case when the Comment Constant is progressing
                else -> return null
            }

        }
    }
}

class CommentTransformWrapper(val mediaVideoUrl : String?,
                              val mediaHeight: Int,
                              var mediaWidth: Int,
                              var mediaTitle: String,
                              var mediaImageUrl: String?,
                              val commentType: String,
                              val otherVideo: Boolean,
                              val mediaId : String, override var isEnabledHD: Boolean) : IMediaWrapperV2, IOverlayable {


    override fun getTitle(): String = mediaTitle

    override fun getId(): String = mediaId

    override fun getImageUrl() = mediaImageUrl

    override fun getImageWidth() = mediaWidth

    override fun getImageHeight() = mediaHeight

    override fun getVideoUrl() = mediaVideoUrl

    // animated is not a gif
    override fun isOtherVideo() = otherVideo

    override fun getMediaType() = commentType
}

