package com.under9.android.comments.model.wrapper

import com.under9.android.lib.blitz.BlitzItem
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.model.CommentUserPrefsState
import com.under9.android.comments.model.User
import com.under9.android.lib.util.L10nUtil
import java.lang.ref.WeakReference
import java.util.*

class UserWrapper private constructor(
        underlyingObject: User
): BlitzItem<User>(underlyingObject), UserItemWrapperInterface {

    /**
     * Default cached accent color is null, but it could be updated manually, and
     * use it instead of underlying object
     * e.g, quick access to edit accent color from post view
     */
    var cachedAccentColor: String? = null

    override fun getUserId(): String = underlyingObject.userId

    override fun getAccountId(): String = underlyingObject.accountId
    
    override fun getName(): String = underlyingObject.displayName

    override fun getEmojiStatus(): String = underlyingObject.emojiStatus

    override fun getCountryEmoji(): String = L10nUtil.countryCodeToEmoji(underlyingObject.country) ?: ""

    override fun getAvatarUrl(): String = underlyingObject.avatarUrl

    override fun isPro(): Boolean = underlyingObject.isActivePro

    override fun isProPlus(): Boolean = underlyingObject.isActiveProPlus

    override fun isActive(): Boolean {
        return (System.currentTimeMillis() - underlyingObject.activeTs * 1000) <= CommentSystem.getInstance().config.activeDuration
    }

    override fun getDisplayName(): String = underlyingObject.displayName

    companion object {
        private val objectMap = WeakHashMap<User, WeakReference<UserWrapper>>()

        @JvmStatic
        fun obtainInstance(user: User): UserItemWrapperInterface {
            synchronized(objectMap) {
                val ref = objectMap[user]
                if (ref != null) {
                    val obj = ref.get()
                    if (obj != null) {
                        return obj
                    }
                }

                val obj = UserWrapper(user)
                objectMap[user] = WeakReference(obj)
                return obj
            }
        }
    }

    override fun isHideProBadge() = underlyingObject.hideProBadge == CommentUserPrefsState.HIDE_PRO_BADGE

    override fun isHideActiveTs() = underlyingObject.hideActiveTs ==  CommentUserPrefsState.HIDE_ACTIVE_TS

    override fun getAccentColor() = cachedAccentColor?.let { it } ?: run { underlyingObject.userAccentColor }
}