package com.under9.android.comments.ui.fragment.dialog

import android.content.Context
import android.util.SparseIntArray
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.commentsystem.R
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_SIMPLE
import com.under9.android.lib.bottomsheet.data.BottomSheetModel
import com.under9.android.lib.bottomsheet.data.BottomSheetMenuItems

object CommentBottomSheetBuilderV2 {

    @JvmStatic
    fun buildActionTextSheet(commentItemWrapper: CommentItemWrapperInterface, context: Context): BottomSheetMenuItems {
        val actionItems = ArrayList<BottomSheetModel>()

        with(context) {
            val taTitle = context.resources.obtainTypedArray(R.array.cs_actionSheetTitles)
            val taIcon = context.resources.obtainTypedArray(R.array.cs_actionSheetIcons)
            val taId = context.resources.obtainTypedArray(R.array.cs_actionSheetIds)
            val mapping = SparseIntArray()

            var shouldShowDeleteButton = commentItemWrapper.isOP && commentItemWrapper.isMyComment() || commentItemWrapper.isMyComment()

            for (i in 0 until taId.length()) {
                val resId = taId.getResourceId(i, -1)
                mapping.put(resId, i)

                val title = getString(taTitle.getResourceId(i, -1))
                val id = taId.getResourceId(i, -1)
                val icon = taIcon.getResourceId(i, -1)

                var shouldAdd = false
                when (id) {
                    R.id.action_report_comment -> {
                        if (!shouldShowDeleteButton) shouldAdd = true
                    }

                    R.id.action_delete_comment -> {
                        if (shouldShowDeleteButton) shouldAdd = true
                    }

                    R.id.action_unfollow_comment -> {
                        if (commentItemWrapper.isFollowed) shouldAdd = true
                    }

                    R.id.action_follow_comment -> {
                        if (!commentItemWrapper.isFollowed) shouldAdd = true
                    }

                    else -> shouldAdd = true
                }

                if (shouldAdd) {
                    actionItems.add(BottomSheetModel(
                            title = title,
                            id = id,
                            iconRes = icon,
                            displayType = ITEM_TYPE_SIMPLE,
                            additionalIconColor = if (id == R.id.action_unfollow_comment) R.color.under9_theme_blue else -1
                    ))
                }
            }

            taTitle.recycle()
            taIcon.recycle()
            taId.recycle()
        }
        return BottomSheetMenuItems(actionItems)
    }

    @JvmStatic
    fun buildActionMediaSheet(context: Context): BottomSheetMenuItems {
        val actionItems = ArrayList<BottomSheetModel>()
        with(context) {

            val taTitle = context.resources.obtainTypedArray(R.array.cs_actionSheetMediaTitles)
            val taIcon = context.resources.obtainTypedArray(R.array.cs_actionSheetMediaIcons)
            val taId = context.resources.obtainTypedArray(R.array.cs_actionSheetMediaIds)
            for (i in 0 until taId.length()) {
                actionItems.add(BottomSheetModel(
                        title = getString(taTitle.getResourceId(i, -1)),
                        id = taId.getResourceId(i, -1),
                        iconRes = taIcon.getResourceId(i, -1),
                        displayType = ITEM_TYPE_SIMPLE
                ))
            }
            taTitle.recycle()
            taIcon.recycle()
            taId.recycle()
        }
        return BottomSheetMenuItems(actionItems)
    }

}