package com.under9.android.comments.ui.renderer

import android.os.Bundle
import android.util.ArrayMap
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.adapter.CommentItemAction
import com.under9.android.comments.adapter.CommentItemClickListenerV2
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.lib.network.UrlMapperInterface

class BubbleCommentRendererDelegate(
        urlMapperInterface: UrlMapperInterface,
        private val appInfoRepository: AppInfoRepositoryInterface,
        renderOptionBundle: Bundle?,
        commentItemActionHandler: CommentItemAction,
        userAccentColorMap: ArrayMap<String, Int>
): CommentItemInfoRendererInterface {

    private val commentItemClickListener: CommentItemClickListenerV2 = CommentItemClickListenerV2(commentItemActionHandler)

    private val renderers: MutableList<BaseCommentItemRenderer> = mutableListOf()

    init {
        renderers.add(BubbleBodyRenderer(commentItemClickListener, renderOptionBundle, userAccentColorMap))
        renderers.add(BubbleVoteInfoRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(BubbleTextContentRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(AvatarRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(BubbleUserInfoRenderer(commentItemClickListener, renderOptionBundle, userAccentColorMap))
        renderers.add(BubbleOpInfoRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(BubbleMediaRenderer(commentItemClickListener, renderOptionBundle, urlMapperInterface))
    }

    override fun bind(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, position: Int) {
        for (i in 0 until renderers.size) {
            renderers[i].bind(level, wrapper, themeAttr, viewHolder, position)
        }
    }

    override fun refreshRenderOptions(renderOptions: Bundle?) {
        for (i in 0 until renderers.size) {
            renderers[i].refreshRenderOptions(renderOptions)
        }
    }
}