package com.under9.android.comments.ui.widget

import android.content.Context
import android.util.AttributeSet
import android.view.View
import android.widget.CheckBox
import androidx.annotation.AttrRes
import androidx.annotation.StyleRes
import com.under9.android.comments.adapter.CommentListItemAdapter.Companion.MODE_BUBBLE_COMMENT_HIDE_ACTION
import com.under9.android.comments.adapter.CommentListItemAdapter.Companion.MODE_BUBBLE_COMMENT_QUICK_ACTION
import com.under9.android.commentsystem.R

class BubbleCommentView : BaseCommentItemView {

    constructor(context: Context): this(context, null, -1, MODE_BUBBLE_COMMENT_HIDE_ACTION)

    constructor(context: Context, bubbleMode: Int): this(context, null, -1, bubbleMode)

    constructor(context: Context, attributeSet: AttributeSet?, bubbleMode: Int) : this(context, attributeSet, -1, bubbleMode)

    constructor(context: Context, @AttrRes attributeSet: AttributeSet?, @StyleRes defStyleAttr: Int, bubbleMode: Int) : super(context, attributeSet, defStyleAttr) {
        attributeSet?.let {
            loadAttributes(context, attributeSet, defStyleAttr)
        }
        this.bubbleMode = bubbleMode
    }
    val bubbleMode: Int
    private lateinit var textBubbleBackground: View
    private lateinit var uivBubbleContainer: View
    private lateinit var circleLikeContainer: View
    private lateinit var circleLikeBtn: CheckBox
    private var isHideInteractionBar = true

    init {
        layoutId = R.layout.cs_list_item_bubble_view
    }

    override fun initView() {
        super.initView()
        textBubbleBackground = findViewById(R.id.textBubbleBackground)
        uivBubbleContainer = findViewById(R.id.uivBubbleContainer)
        circleLikeBtn = findViewById(R.id.circleLikeBtn)
        circleLikeContainer = findViewById(R.id.circleLikeContainer)

        hideInteractionBar()
        if (bubbleMode != MODE_BUBBLE_COMMENT_QUICK_ACTION) {
            circleLikeContainer.visibility = View.GONE
            circleLikeBtn.visibility = View.GONE
        } else {
            circleLikeContainer.visibility = View.VISIBLE
            circleLikeBtn.visibility = View.VISIBLE
        }
    }

    // if the mode is hide by default, click will have hiding/ showing interaction bar
    fun hideShowInteractionBar() {
        isHideInteractionBar = if (isHideInteractionBar) {
            showInteractionBar()
            false
        } else {
            hideInteractionBar()
            true
        }
    }

    private fun hideInteractionBar() {
        replyBtn.visibility = View.GONE
        likeBtn.visibility = View.GONE
        dislikeBtn.visibility = View.GONE
        moreBtn.visibility = View.GONE
    }

    private fun showInteractionBar() {
        replyBtn.visibility = View.VISIBLE
        likeBtn.visibility = View.VISIBLE
        dislikeBtn.visibility = View.VISIBLE
        moreBtn.visibility = View.VISIBLE
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        // TODO: should store the hide/show status on the data model
        if (!isHideInteractionBar) hideInteractionBar()
    }


}