package com.under9.android.lib.blitz

import com.under9.android.lib.blitz.adapter.BlitzItemAdapter
import timber.log.Timber

internal class BlitzItemAdapterDefaultListener<T>(
        private val blitz: Blitz<T>
) : Blitz.LoaderListener<T> {

    lateinit var adapter: BlitzItemAdapter<T>

    override fun onInit() {
        Timber.d("onInit")
    }

    override fun onStartRefresh() {
        Timber.d("onStartRefresh")
    }

    override fun onInitDone(items: MutableList<T>, hasNext: Boolean, hasPrev: Boolean, listMeta: MutableMap<String, String>?) {
        adapter.notifyDataSetChanged()
        Timber.d("onInitDone: " + blitz.javaClass.name + "@" + Integer.toHexString(blitz.hashCode()) + ", listSize=" + blitz.size + ", delta=" + items!!.size)
    }

    override fun onRefreshDone(items: MutableList<T>, hasNext: Boolean, listMeta: MutableMap<String, String>?) {
        adapter.notifyDataSetChanged()
        Timber.d("onRefreshDone")
    }

    override fun onLoadNextDone(items: MutableList<T>, hasNext: Boolean, appendPosition: Int) {
        Timber.d("onLoadNextDone: getItemCount=" + adapter.itemCount + ", itemsSize=" + items.size)
        adapter.notifyItemRangeInserted(appendPosition, items.size)
    }

    override fun onLoadPrevDone(items: MutableList<T>, hasPrev: Boolean, prependPosition: Int) {
        Timber.d("onLoadPrevDone: getItemCount=" + adapter.itemCount + ", itemsSize=" + items.size)
        adapter.notifyItemRangeInserted(0, items.size)
    }

    override fun onRefreshError(throwable: Throwable?) {
        Timber.e(throwable, "onRefreshError")
    }

    override fun onLoadNextError(throwable: Throwable?) {
        Timber.e(throwable, "onLoadNextError")
    }
}