package com.under9.android.lib.blitz.adapter

import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.annotation.LayoutRes
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.Guideline
import com.ninegag.android.blitz.R
import com.under9.android.lib.blitz.PlaceholderState

/**
 * This is a very simple and generic placeholder,
 * with a description text and an aciton button only.
 *
 * You may implement your own placeholder adapter by extending [BaseToggleableAdapter]
 * Created by raymond on 8/8/2016.
 */

class BlitzPlaceholderAdapterV2 : BaseToggleableAdapter() {
    lateinit var config: Config
    private var currState: PlaceholderState = PlaceholderState.UninitializedState(false, true)

    private var clickListener: View.OnClickListener? = null

    private class PlaceholderActionClickListener internal constructor(private val actionListener: ActionClickListener?) : View.OnClickListener {

        override fun onClick(v: View) {
            val id = v.id

            if (actionListener == null) return

            if (id == R.id.blitzStateActionButton) {
                actionListener.invoke(id)
            } else if (id == R.id.blitzEmptyLabel) {

            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BlitzSubAdapter.ViewHolder {
        if (config.layoutId == -1) throw IllegalArgumentException("Layout Id cannot be -1")

        val view = LayoutInflater.from(parent.context).inflate(config.layoutId, parent, false)
        clickListener = PlaceholderActionClickListener(config.listener)
        return ViewHolder(view)
    }

    override fun onBindViewHolder(holder: BlitzSubAdapter.ViewHolder, position: Int) {
        super.onBindViewHolder(holder, position)
        val vh = holder as ViewHolder
        vh.bind(clickListener, config, currState)
    }

    private class ViewHolder(itemView: View) : BlitzSubAdapter.ViewHolder(itemView) {
        /**
         * Title of the placeholder
         */
        internal var placeholderTitleText: TextView? = null

        /**
         * Some description of the placeholder
         */
        internal var blitzPlaceHolderDesc: TextView? = null
        /**
         * An action button on placeholder
         */
        internal var actionButton: TextView? = null
        internal var blitzErrorLabel: TextView? = null

        // view groups
        internal var blitzErrorContainer: ViewGroup = itemView.findViewById(R.id.blitzErrorContainer)
        internal var blitzEmptyTitleContainer: ViewGroup = itemView.findViewById(R.id.blitzEmptyTitleContainer)
        internal var blitzEmptyTitleDescContainer: ViewGroup = itemView.findViewById(R.id.blitzEmptyTitleDescContainer)
        internal var blitzLoadingContainer: ViewGroup = itemView.findViewById(R.id.blitzLoadingContainer)
        internal var blitzPlaceHolderContainer: ViewGroup = itemView.findViewById(R.id.blitzPlaceHolderContainer)
        internal var centerHorizontalGuideline: Guideline = itemView.findViewById(R.id.centerHorizontalGuideline)

        init {
            placeholderTitleText = itemView.findViewById(R.id.blitzPlaceholderTextTitle)
            blitzPlaceHolderDesc = itemView.findViewById(R.id.blitzPlaceHolderDesc)
            actionButton = itemView.findViewById(R.id.blitzStateActionButton)
            blitzErrorLabel = itemView.findViewById(R.id.blitzErrorLabel)
        }

        fun bind(clickListener: View.OnClickListener?,
                 config: Config,
                 state: PlaceholderState) {

            hideAllLoadingView()
            hideAllErrorEmptyView()

            when (state) {
                is PlaceholderState.UninitializedState -> return

                is PlaceholderState.SuccessState -> {
                }

                is PlaceholderState.LoadingState -> {
                    blitzLoadingContainer.visibility = View.VISIBLE
                }

                is PlaceholderState.EmptyState -> {
                    if (!config.placeholderDescriptionText.isNullOrEmpty()) {
                        // if have description, use multiple lines with desc
                        blitzEmptyTitleDescContainer.visibility = View.VISIBLE
                    } else {
                        // if no desc input before, use single line empty state
                        blitzEmptyTitleContainer.visibility = View.VISIBLE
                    }

                    if (!config.placeholderDescriptionText.isNullOrEmpty()) {
                        blitzPlaceHolderDesc?.let {
                            it.visibility = View.VISIBLE
                            it.text = config.placeholderDescriptionText
                        }
                    }

                    if (!config.placeholderTitleText.isNullOrEmpty()) {
                        placeholderTitleText?.let {
                            it.visibility = View.VISIBLE
                            it.text = config.placeholderTitleText
                        }
                    }
                }

                is PlaceholderState.ErrorState -> {
                    blitzErrorContainer.visibility = View.VISIBLE

                    if (!config.actionLabelText.isNullOrEmpty()) {
                        actionButton?.let {
                            it.visibility = View.VISIBLE
                            it.text = config.actionLabelText
                            it.setOnClickListener(clickListener)
                        }
                    }

                    if (!config.errorText.isNullOrEmpty()) {
                        blitzErrorLabel?.let {
                            it.visibility = View.VISIBLE
                            it.text = config.errorText
                        }
                    }
                }
            }
            changePlaceholderSizeByState(state)
        }

        private fun changePlaceholderSizeByState(state: PlaceholderState) {
            if (state.showAsFullscreen) {
                centerHorizontalGuideline.setGuidelinePercent(0.4F)
                blitzPlaceHolderContainer.layoutParams = ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.MATCH_PARENT, ConstraintLayout.LayoutParams.MATCH_PARENT)
            } else {
                centerHorizontalGuideline.setGuidelinePercent(0.5F)
                blitzPlaceHolderContainer.layoutParams = ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.MATCH_PARENT, ConstraintLayout.LayoutParams.WRAP_CONTENT)
            }
        }

        private fun hideAllLoadingView() {
            blitzLoadingContainer.visibility = View.GONE
        }

        private fun hideAllErrorEmptyView() {
            blitzErrorContainer.visibility = View.GONE
            blitzEmptyTitleContainer.visibility = View.GONE
            blitzErrorContainer.visibility = View.GONE
            blitzEmptyTitleContainer.visibility = View.GONE
            blitzEmptyTitleDescContainer.visibility = View.GONE
        }
    }

    fun setPlaceholderState(state: PlaceholderState) {
        // No point to update if both have the same value
        if (currState == state) return
        this.currState = state
        isEnable = currState.enabled
        notifyDataSetChanged()
    }

    class Builder private constructor() {
        private var actionLabelText: String? = null
        private var placeholderDescriptionText: String? = null
        private var placeholderTitleText: String? = null
        private var errorText: String? = null
        @LayoutRes
        private var layoutId = -1
        @LayoutRes
        private var emptyPlaceholderLayoutId = -1
        private var actionListener: ActionClickListener? = null

        fun setActionLabelText(actionLabelText: String): Builder {
            this.actionLabelText = actionLabelText
            return this
        }

        fun setLayoutId(layoutId: Int): Builder {
            this.layoutId = layoutId
            return this
        }

        fun setEmptyPlaceholderLayoutId(emptyPlaceholderLayoutId: Int): Builder {
            this.emptyPlaceholderLayoutId = emptyPlaceholderLayoutId
            return this
        }

        fun setActionListener(actionListener: ActionClickListener): Builder {
            this.actionListener = actionListener
            return this
        }

        fun setPlaceholderTitleText(placeholderTitleText: String): Builder {
            this.placeholderTitleText = placeholderTitleText
            return this
        }

        fun setPlaceholderDescriptionText(placeholderDescriptionText: String): Builder {
            this.placeholderDescriptionText = placeholderDescriptionText
            return this
        }

        fun setErrorText(errorText: String): Builder {
            this.errorText = errorText
            return this
        }

        fun build(): BlitzPlaceholderAdapterV2 {
            val adapter = BlitzPlaceholderAdapterV2()
            adapter.config = Config(
                    actionLabelText = actionLabelText,
                    layoutId = layoutId,
                    emptyPlaceholderLayoutId = emptyPlaceholderLayoutId,
                    listener = actionListener,
                    placeholderDescriptionText = placeholderDescriptionText,
                    placeholderTitleText = placeholderTitleText,
                    errorText = errorText)
            return adapter
        }

        companion object {
            fun newBuilder(): Builder {
                return Builder()
            }
        }
    }

    override fun getItemViewType(position: Int): Int {
        return R.id.blitz_view_type_placeholder_adapter
    }

    fun updatePlaceholderDescription(text: String) {
        config.placeholderDescriptionText = text
        notifyItemChanged(0)
    }
}

typealias ActionClickListener = (Int) -> Unit

/**
 * A listener for placeholder's action
 */
data class Config(
        val listener: ActionClickListener? = null,
        val actionLabelText: String? = null,
        var placeholderDescriptionText: String? = null,
        val placeholderTitleText: String? = null,
        val errorText: String? = null,
        @LayoutRes
        val layoutId: Int = -1,
        @LayoutRes
        val emptyPlaceholderLayoutId: Int = -1)
