package com.under9.android.lib.blitz.delegate

import android.widget.LinearLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.blitz.BlitzActionListener
import com.under9.android.lib.blitz.BlitzState
import com.under9.android.lib.blitz.BlitzView
import timber.log.Timber

/**
 * Created by raymond on 10/8/2016.
 */

class LinearPaginationDetector @JvmOverloads constructor(private val actionListener: BlitzActionListener,
                                                         loadMoreBound: Int = DEFAULT_LOAD_MORE_BOUND, shouldTriggeredOnAttachment: Boolean = true) : PaginationDetector {

    private var orientation = VERTICAL
    private var loadMoreBound = DEFAULT_LOAD_MORE_BOUND // default 20
    private var shouldTriggeredOnAttachment = true

    private var lastTriggerLoadNext: Long = 0
    private var lastTriggerLoadPrev: Long = 0

    init {
        this.loadMoreBound = loadMoreBound
        this.shouldTriggeredOnAttachment = shouldTriggeredOnAttachment
    }

    override fun checkPagination(blitzView: BlitzView, layoutManager: RecyclerView.LayoutManager?, dx: Int, dy: Int) {
        val linearLayoutManager = layoutManager as LinearLayoutManager?
        if (VERTICAL == orientation) {
            checkLoadVertical(blitzView, linearLayoutManager, dy)
        } else {
            checkLoadHorizontal(blitzView, linearLayoutManager, dx)
        }
    }

    /**
     * By default, [android.support.v7.widget.RecyclerView.OnScrollListener.onScrolled]
     * will be triggered after scroll listener is added, we use this to configure
     * if we want this behaviour
     */
    override fun shouldTriggerOnAttachment(): Boolean {
        return this.shouldTriggeredOnAttachment
    }

    private fun checkLoadVertical(blitzView: BlitzView, linearLayoutManager: LinearLayoutManager?, dy: Int) {

        val now = System.currentTimeMillis()
        val viewState = blitzView.viewState

        if (linearLayoutManager == null) return
        if (viewState == BlitzState.STATE_INIT_LOADING ||
                viewState == BlitzState.STATE_REFRESHING ||
                viewState == BlitzState.STATE_LOADING_MORE)
            return

        val itemCount = linearLayoutManager.itemCount
        val visibleItemCount = linearLayoutManager.childCount
        val firstVisibleItem = linearLayoutManager.findFirstVisibleItemPosition()

        val offset = if (loadMoreBound <= itemCount) loadMoreBound else loadMoreBound / 2
        Timber.d("checkLoadVertical: itemCount=" + itemCount +
                "\n visibleItemCount=" + visibleItemCount +
                "\nfirstVisibleItem=" + firstVisibleItem +
                "\ndy = $dy , offset $offset")
        if (dy > 0 && firstVisibleItem + visibleItemCount >= offset) {
            if (now - lastTriggerLoadNext < LOAD_MORE_CHECK_RATE_LIMIT) return
            if (viewState == BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST) return
            if (actionListener.onTriggerLoadNext() && linearLayoutManager.stackFromEnd) {
                linearLayoutManager.stackFromEnd = false
            }
            lastTriggerLoadNext = now

            Timber.d("checkLoadVertical: --loadNext=$lastTriggerLoadNext")
        }

        if (dy < 0 && firstVisibleItem == 0) {
            if (now - lastTriggerLoadPrev < LOAD_MORE_CHECK_RATE_LIMIT) return
            if (viewState == BlitzState.STATE_FINISH_LOADING_PREV_END_OF_LIST) return
            if (actionListener.onTriggerLoadPrevious() && !linearLayoutManager.stackFromEnd) {
                linearLayoutManager.stackFromEnd = true
            }
            lastTriggerLoadPrev = now

            Timber.d("checkLoadVertical: --loadPrev=$lastTriggerLoadPrev")
        }
    }

    private fun checkLoadHorizontal(blitzView: BlitzView, linearLayoutManager: LinearLayoutManager?, dx: Int) {
        // TODO: 10/8/2016
    }

    fun setOrientation(orientation: Int) {
        this.orientation = orientation
    }

    companion object {

        private val TAG = "LinearPaginationDetecto"
        val VERTICAL = LinearLayout.VERTICAL
        val HORIZONTAL = LinearLayout.HORIZONTAL

        private val LOAD_MORE_CHECK_RATE_LIMIT: Long = 1000
        private val DEFAULT_LOAD_MORE_BOUND = 20
    }
}
