package com.under9.android.lib.bottomsheet.action

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import com.under9.android.lib.bottomsheet.StyledBottomSheetDialogFragment
import com.under9.android.lib.util.dpToPx
import com.under9.android.lib.widget.R
import com.under9.android.lib.widget.button.LoadingButton
import com.under9.android.lib.widget.radiobutton.ImageTitleRadioButtonView
import com.under9.android.lib.widget.radiobutton.SelectionRadioGroup
import kotlinx.android.synthetic.main.view_selection_bottom_sheet.*
import kotlinx.android.synthetic.main.view_selection_radio_group.view.*

class SelectionBottomSheet : StyledBottomSheetDialogFragment() {

    private lateinit var model: ISelectionSheetModel
    private var listener: ISelectionClickListener? = null
    private var isTypeLoadableSheet = false
    companion object {
        const val KEY_SELECTION_MODEL = "selection_model"
        const val KEY_TYPE_LOADABLE_SHEET = "type_loadable_sheet"

        fun newInstance(model: ISelectionSheetModel, isLoadable: Boolean): SelectionBottomSheet {
            val args = Bundle()
            args.putParcelable(KEY_SELECTION_MODEL, model)
            args.putBoolean(KEY_TYPE_LOADABLE_SHEET, isLoadable)
            val frag = SelectionBottomSheet()
            frag.arguments = args
            return frag
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let { it ->
            it.getParcelable<ISelectionSheetModel>(KEY_SELECTION_MODEL)?.let { model ->
                this.model = model
            }
            isTypeLoadableSheet = it.getBoolean(KEY_TYPE_LOADABLE_SHEET, false)
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.view_selection_bottom_sheet, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        // set icon
        if (!::model.isInitialized) return
        with(actionSheetIcon) {
            model.iconRes?.let {
                background = view.context.getDrawable(it)
            } ?: run {
                visibility = View.GONE
            }
        }

        // set title
        title.text = model.title

        // set desc
        with(desc) {
            model.desc?.let {
                text = model.desc
            } ?: run {
                visibility = View.GONE
            }
        }

        if (!isTypeLoadableSheet) {
            // set primary button color
            with(primaryButton) {
                text = model.primaryActionText
                background = ContextCompat.getDrawable(context, model.primaryButtonBgDrawableRes)
                setTextColor(ContextCompat.getColor(context, model.primaryButtonTextColor))
                setOnClickListener {
                    dismiss()
                    listener?.onPrimaryClick(this@SelectionBottomSheet)
                }
            }

            with(secondaryButton) {
                model.secondaryActionText?.let {
                    text = it
                    background = ContextCompat.getDrawable(context, model.secondaryButtonBgDrawableRes!!)
                    if (model.secondaryButtonTextColor != null) {
                        setTextColor(ContextCompat.getColor(context, model.secondaryButtonTextColor!!))
                    }
                    setOnClickListener {
                        dismiss()
                        listener?.onSecondaryClick(this@SelectionBottomSheet)
                    }
                } ?: run {
                    visibility = View.GONE

                    // if second is null, primary should fill width
                    val primaryButtonParam = primaryButton.layoutParams as ConstraintLayout.LayoutParams
                    primaryButtonParam.leftToLeft = sheetRootView.id
                    primaryButtonParam.marginEnd = 16.dpToPx(context)
                    primaryButtonParam.rightMargin = 16.dpToPx(context)
                    primaryButtonParam.leftMargin = 16.dpToPx(context)
                    primaryButtonParam.leftMargin = 16.dpToPx(context)
                    primaryButton.requestLayout()
                }
            }

            loadingButton.visibility = View.GONE
        } else {
            primaryButton.visibility = View.GONE
            secondaryButton.visibility = View.GONE
            loadingButton.visibility = View.VISIBLE
            loadingButton.setOnClickListener {
                loadingButton.currentState = LoadingButton.State.Loading
                listener?.onPrimaryClick(this@SelectionBottomSheet)
            }
        }

        if (model is ChoiceBottomSheetModel) {
            radioGroupContainer.visibility = View.VISIBLE
            val rg = radioGroupContainer.radioGroup as SelectionRadioGroup
            (model as ChoiceBottomSheetModel).choices.forEach {
                val radioView = ImageTitleRadioButtonView(context!!)
                radioView.updateContent(it)
                rg.addView(radioView)
            }
            rg.setRadioGroupSelectedListener {
                if (isTypeLoadableSheet) {
                    loadingButton.currentState = LoadingButton.State.Enabled
                }
            }
        }
    }

    fun getSelectedPosition(): Int? = (radioGroupContainer.radioGroup as SelectionRadioGroup).getSelectedPosition()

    fun setSelectionClickListener(listener: ISelectionClickListener) {
        this.listener = listener
    }
}

interface ISelectionClickListener {
    fun onPrimaryClick(sheet: SelectionBottomSheet)
    fun onSecondaryClick(sheet: SelectionBottomSheet)
}


