package com.under9.android.lib.bottomsheet.color

import android.graphics.PorterDuff
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.bottomsheet.Position
import com.under9.android.lib.widget.R
import kotlinx.android.synthetic.main.view_bottom_sheet_item_color.view.*

class BottomSheetColorPickerAdapter(private val items: List<ColorPickerModel>,
                                    private val clickListener: ColorPickerSheetClickListener): RecyclerView.Adapter<BottomSheetColorPickerAdapter.VH>(){

    override fun onBindViewHolder(holder: VH, position: Int) {
        val model = items[position]
        with(holder.itemView) {
            colorName.text = model.title
            if (items[position].isDefaultColor) {
                colorView.background = holder.itemView.context.getDrawable(R.drawable.color_circle)
            } else {
                colorView.background.setColorFilter(items[position].colorInt, PorterDuff.Mode.SRC_ATOP)
            }

            if (model.isLocked) {
                featureLockIcon.visibility = View.VISIBLE
            } else {
                featureLockIcon.visibility = View.GONE
            }

            if (model.isChecked) {
                selectedRow.visibility = View.VISIBLE
            } else {
                selectedRow.visibility = View.GONE
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): VH {
        return VH(LayoutInflater.from(parent.context).inflate(R.layout.view_bottom_sheet_item_color, parent, false), items, clickListener)
    }

    override fun getItemCount() = items.size

    class VH(itemView: View,
             items: List<ColorPickerModel>,
             clickListener: ColorPickerSheetClickListener): RecyclerView.ViewHolder(itemView) {
        init {
            itemView.setOnClickListener {
                clickListener.invoke(adapterPosition, items[adapterPosition].title)
            }
        }
    }
}
typealias ColorName = String
typealias ColorPickerSheetClickListener = ((Position), (ColorName)) -> Unit
