package com.under9.android.lib.bottomsheet.color

import android.app.Dialog
import android.os.Build
import android.os.Bundle
import android.view.ViewGroup
import android.widget.FrameLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.behavior.BedModeController
import com.under9.android.lib.bottomsheet.GagBottomSheetDialogFragment
import com.under9.android.lib.bottomsheet.Position
import com.under9.android.lib.bottomsheet.StyledBottomSheetDialogFragment
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.view.rootview.GagFrameLayout
import com.under9.android.lib.widget.R

class ColorBottomSheetDialogFragment: StyledBottomSheetDialogFragment() {

    private lateinit var items: List<ColorPickerModel>
    private var listener: ColorPickerSheetClickListener = { _: Position, _: ColorName -> }

    companion object {
        private const val KEY_MENU_ITEMS = "menu_items"

        const val FRAG_TAG = "more_action"
        const val DEFAULT_COLOR = Int.MIN_VALUE

        @JvmStatic
        fun newInstance(items: ColorPickerListModel, isEnabledBedMode: Boolean): ColorBottomSheetDialogFragment {
            val args = Bundle()
            args.putParcelable(KEY_MENU_ITEMS, items)
            args.putBoolean(GagBottomSheetDialogFragment.KEY_BED_MODE_ENABLED, isEnabledBedMode)
            val frag = ColorBottomSheetDialogFragment()
            frag.arguments = args
            return frag
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val listModel = arguments?.getParcelable<ColorPickerListModel>(KEY_MENU_ITEMS)
        listModel?.items.let { this.items = it!! }
        isEnabledBedModel = arguments?.getBoolean(GagBottomSheetDialogFragment.KEY_BED_MODE_ENABLED, false)!!

    }

    fun setBottomSheetClickListener(listener: ColorPickerSheetClickListener){
        this.listener = listener
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = super.onCreateDialog(savedInstanceState)
        val matchAllParentParam = FrameLayout.LayoutParams(ViewGroup.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT))

        val container = GagFrameLayout(context!!).apply {
            // Currently, just apply the change of dark theme for the version >= LOLLIPOP, because it make codes very messy if handle pre-lollipop here,
            // And it will drop pre-lollipop later, so TODO: clean up when dropped pre-lollipop support
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                setBackgroundColor(UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1))
            }
            layoutParams = matchAllParentParam
            id = R.id.more_menu_button_sheet
        }

        val rv = RecyclerView(context!!).apply {
            layoutParams =  FrameLayout.LayoutParams(ViewGroup.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT)).apply {
                bottomMargin = context!!.resources.getDimension(R.dimen.space8).toInt()
            }
            id = R.id.more_menu_recyclerview
            layoutManager = LinearLayoutManager(context)
            adapter = BottomSheetColorPickerAdapter(items) { pos: Int, colorName: ColorName ->
                listener.invoke(pos, colorName)
                dialog.dismiss()
            }
            hasFixedSize()
        }

        if (isEnabledBedModel) {
            BedModeController(context!!, false, false).apply {
                registerCanvasView(container)
                disableChangeNavColor()
                enableBedMode()
            }
        }
        container.addView(rv)
        dialog.setContentView(container)
        return dialog
    }
}