package com.under9.android.lib.morpheus.ui

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.Checkable
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.core.view.ViewCompat
import com.under9.android.lib.morpheus.R
import java.util.*

open class NotifAdapter(list: List<AbstractNotifItemWrapper?>?) : BaseAdapter() {
    private val list: MutableList<AbstractNotifItemWrapper> = Collections.synchronizedList(list)
    override fun getCount(): Int {
        return list.size
    }

    fun addItem(item: AbstractNotifItemWrapper) {
        list.add(item)
    }

    override fun getItem(position: Int): Any? {
        return null
    }

    override fun getItemId(position: Int): Long {
        return 0
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
        var localConvertView = convertView
        val holder: ViewHolder
        val item = list[position]
        if (localConvertView == null) {
            val inflater = parent.context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
            localConvertView = inflater.inflate(layoutId, parent, false)
            holder = createViewHolder(localConvertView)
            localConvertView.setTag(holder)
            val drawable = ContextCompat.getDrawable(localConvertView.getContext(), R.drawable.morpheus_notif_item)
            ViewCompat.setBackground(localConvertView, drawable)
        } else {
            holder = localConvertView.tag as ViewHolder
        }
        fillContent(parent.context, holder, item)
        return localConvertView!!
    }

    protected open fun createViewHolder(convertView: View): ViewHolder {
        return ViewHolder(convertView)
    }

    protected open val layoutId: Int
        protected get() = R.layout.morpheus_notif_item

    protected open fun fillContent(context: Context?, holder: ViewHolder?, item: AbstractNotifItemWrapper?) { // https://fabric.io/9gag2/android/apps/com.ninegag.android.app/issues/563075c9f5d3a7f76b220bb7
        if (holder == null || item == null) return
        holder.text.text = item.getRenderedText(context)
        holder.text.setOnClickListener(item.getOnClickListener())
        holder.timeText.text = item.getTimeText(context)
        //        holder.container.setSelected(true);
        if (holder.thumbRightUIV != null) {
            if (!item.hasThumbnailRight()) {
                holder.thumbRightUIV!!.visibility = View.GONE
            } else {
                holder.thumbRightUIV!!.visibility = View.VISIBLE
            }
        }
        holder.container.setOnClickListener(item.getOnClickListener())
        // highlighted = unread
        // normal = read
        if (holder.container is Checkable) {
            (holder.container as Checkable).isChecked = item.isRead
        }
        if (holder.text is Checkable) {
            (holder.text as Checkable).isChecked = item.isRead
        }
    }

    open class ViewHolder(view: View?) {
        var container: View
        var text: TextView
        var timeText: TextView
        var thumbLeftUIV: View
        var thumbRightUIV: View?

        init {
            container = view!!.findViewById(R.id.container)
            text = view.findViewById(R.id.text)
            timeText = view.findViewById(R.id.timeText)
            /**
             * FIXME: 26/6/2017 The reason for checking instance here is because we are migrating to uiv ver.2
             */
            thumbLeftUIV = view.findViewById(R.id.thumbLeftUIV)
            thumbRightUIV = view.findViewById(R.id.thumbRightUIV)
        }
    }

    companion object {
        private const val TAG = "NotifAdapter"
    }

}