package com.under9.android.lib.widget

import android.content.Context
import android.os.Bundle
import android.util.AttributeSet
import android.view.KeyEvent
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputConnection
import android.widget.EditText

import androidx.appcompat.widget.AppCompatEditText
import androidx.core.os.BuildCompat
import androidx.core.view.inputmethod.EditorInfoCompat
import androidx.core.view.inputmethod.InputConnectionCompat
import androidx.core.view.inputmethod.InputContentInfoCompat

import com.under9.android.lib.util.BundleUtil

import timber.log.Timber

/**
 * Created by raymond on 23/10/2015.
 */
class KeyboardEventEditText : AppCompatEditText {

    private var commitContentListener: InputConnectionCompat.OnCommitContentListener? = null
    private var mEditTextKeyboardEventListener: KeyboardEventEditTextListener? = null

    constructor(context: Context) : super(context) {}

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {}

    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {}

    override fun onKeyPreIme(keyCode: Int, event: KeyEvent): Boolean {
        if (event.keyCode == KeyEvent.KEYCODE_BACK) {
            if (mEditTextKeyboardEventListener != null) {
                mEditTextKeyboardEventListener!!.onImeBack(this)
            }
        }
        return super.onKeyPreIme(keyCode, event)
    }

    override fun onCreateInputConnection(editorInfo: EditorInfo): InputConnection? {
        val ic = super.onCreateInputConnection(editorInfo)
        EditorInfoCompat.setContentMimeTypes(editorInfo, arrayOf("image/*"))
        return if (commitContentListener == null) {
            Timber.i("onCreateInputConnection: null listener, create with original")
            ic
        } else {
			if (ic == null) {
				return null
			} else {
            	Timber.i("onCreateInputConnection: hasListener, listener= $commitContentListener")
           		return InputConnectionCompat.createWrapper(ic, editorInfo, commitContentListener!!)
			}
        }
    }

    fun setKeyboardEventEditTextListener(listener: KeyboardEventEditTextListener?) {
        this.mEditTextKeyboardEventListener = listener
    }

    /**
     * How to handle events from [InputConnection]
     * @param listener
     */
    fun setCommitContentListener(listener: InputConnectionCompat.OnCommitContentListener?) {
        this.commitContentListener = listener
    }

    interface KeyboardEventEditTextListener {
        fun onImeBack(keyboardEventEditText: KeyboardEventEditText)
    }
}
