package com.under9.android.lib.widget

import android.content.Context
import android.graphics.PorterDuff
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.core.graphics.drawable.DrawableCompat
import androidx.core.view.ViewCompat
import com.google.android.material.snackbar.Snackbar
import com.under9.android.lib.util.UiUtil

fun Snackbar.applyRoundedCornerStyle(context: Context) {
    setActionTextColor(UiUtil.themeAttributeToColor(R.attr.under9_themeColorAccent, context, -1))
    val dp8 = context.resources.getDimension(R.dimen.space8).toInt()

    view.findViewById<TextView>(com.google.android.material.R.id.snackbar_text).apply {
        maxLines = 3
        val tvParams = layoutParams as ViewGroup.MarginLayoutParams
        tvParams.height = ViewGroup.LayoutParams.WRAP_CONTENT
        setPadding(dp8, 0, dp8, 0)
        layoutParams = tvParams
    }

    (this.view.layoutParams as ViewGroup.MarginLayoutParams).apply {
        val dp12 = context.resources.getDimension(R.dimen.space12).toInt()
        setMargins(dp12, 0, dp12, dp8)
        height = ViewGroup.LayoutParams.WRAP_CONTENT
    }.also {
        view.layoutParams = it
    }

    view.background = ContextCompat.getDrawable(context, R.drawable.bg_snackbar_dark)

    ViewCompat.setElevation(this.view, 6f)
}

fun ImageView.applyTintColor(color: Int) {
    var d = this.drawable
    d.mutate()
    d = DrawableCompat.wrap(d)

    DrawableCompat.setTint(d, color)
    DrawableCompat.setTintMode(d, PorterDuff.Mode.SRC_IN)
}