package com.under9.android.lib.widget.radiobutton

import android.content.Context
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout

class SelectionRadioGroup: LinearLayout {
    constructor(context: Context?) : super(context) {}
    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs) {}
    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {}

    init {
        layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT)
        orientation = VERTICAL
    }
    private var buttonCount = 0
    private var listener: SelectionRadioGroupListener? = null

    override fun addView(child: View, index: Int, params: ViewGroup.LayoutParams) {
        child.tag = "RADIO_GROUP_PREFIX$buttonCount"

        (child as? ImageTitleRadioButtonView)?.setOnClickListener { view: View? ->
            val selectedButton: ImageTitleRadioButtonView = child
            if (selectedButton.isSelectedItem) return@setOnClickListener

            setAllButtonsToUnselectedState()
            selectedButton.isSelectedItem = true
            listener?.invoke(selectedButton)
        }
        super.addView(child, index, params)
        ++buttonCount
    }

    fun setRadioGroupSelectedListener(listener: SelectionRadioGroupListener) {
        this.listener = listener
    }

    fun getSelectedPosition(): Int? {
        for (i in 0 until childCount) {
            val child = getChildAt(i)
            if (child is ImageTitleRadioButtonView) {
                if (child.isSelectedItem) return i
            }
        }

        return null
    }

    private fun setAllButtonsToUnselectedState() {
        for (i in 0 until childCount) {
            val child = getChildAt(i)
            if (child is ImageTitleRadioButtonView) {
                child.isSelectedItem = false
            }
        }
    }

    companion object {
        const val RADIO_GROUP_PREFIX = "under9_radio_group_tag_"
    }

}
typealias SelectionRadioGroupListener = (View) -> Unit