package com.under9.android.lib.widget.text

import android.content.Context
import android.util.AttributeSet
import androidx.annotation.AttrRes
import androidx.annotation.StyleRes
import androidx.appcompat.widget.AppCompatTextView
import android.text.*
import android.view.MotionEvent
import android.text.style.ClickableSpan
import android.text.style.BackgroundColorSpan
import androidx.annotation.NonNull

/**
 * @ref https://stackoverflow.com/questions/30452627/redirect-touch-event-on-textview-with-clickablespan-to-parent-view
 *
 * This TextView aims to solve the problem where when {@link android.text.method.LinkMovementMethod} is set, parent view's
 * click effect (ripple) is not working
 *
 * @see android.text.method.LinkMovementMethod
 * @see
 */
class ClickableSpanTouchEventRedirectTextView : AppCompatTextView {
    private var spannable: Spannable? = null
    private var backgroundColorSpan: BackgroundColorSpan? = null


    constructor(context: Context): this(context, null, -1)

    constructor(context: Context, attributeSet: AttributeSet?) : this(context, attributeSet, -1)

    constructor(context: Context, @AttrRes attributeSet: AttributeSet?, @StyleRes defStyleAttr: Int) : super(context, attributeSet, defStyleAttr)

    override fun onTouchEvent(@NonNull event: MotionEvent): Boolean {
        val layout = this.layout
        if (layout != null) {
            val line = layout.getLineForVertical(event.y.toInt())
            val offset = layout.getOffsetForHorizontal(line, event.x)

            if (text != null && text is Spanned) {
                val spanned = text as Spanned

                val links = spanned.getSpans(offset, offset, ClickableSpan::class.java)

                // We have to make sure when the text view has some set Spannable
                if (links.isEmpty()) return super.onTouchEvent(event)

                when(event.action) {
                    MotionEvent.ACTION_DOWN -> {
                        spannable = SpannableString(spanned)

                        val start = (text as Spanned).getSpanStart(links[0])
                        val end = (text as Spanned).getSpanEnd(links[0])

                        // We highlight the spannable when pressed (Giving it a background)
                        backgroundColorSpan = BackgroundColorSpan(highlightColor)
                        spannable?.setSpan(backgroundColorSpan, start, end, Spannable.SPAN_INCLUSIVE_INCLUSIVE)

                        Selection.setSelection(spannable, start, end)
                        text = spannable
                        return true
                    }

                    MotionEvent.ACTION_UP -> {
                        links[0].onClick(this)

                        // We remove the background when finger leaves
                        spannable?.let {
                            it.removeSpan(backgroundColorSpan)
                            Selection.removeSelection(it)
                            text = it
                        }
                    }
                    else -> return super.onTouchEvent(event)
                }
            }
        }

        return super.onTouchEvent(event)
    }
}